## Copyright (C) 2018 - Patrick Troxler
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program. If not, see <http://www.gnu.org/licenses/>.

## Author: Patrick Troxler <patrick-troxler@hotmail.com>
## Created: 2018-06-21

## -*- texinfo -*-
## @defun {[@var{delay} @var{maxcorr}]} = finddelay (@var{q}, @var{r}, @var{maxlag}=[], @var{dt}=1)
## Delay between flow and rain signals using their maximum cross-correlation
##
## @var{q} vector of flows q
##
## @var{r} vector of rains r
##
## @var{maxlag} maximum lag to be returned (when exceedeing maxlag)
##
## @var{dt} time step size of vectors q and r
##
## @var{delay} returned delay (based on max. cross corr. of time series)
##
## @var{maxcorr} corresponding maximum cross correlation  
##
## @seealso{xcorr}
## @end defun

function [delay maxcorr]  = finddelay (q, r, maxlag=[], dt=1)

  # Parse arguments
  # if maxlag is not given we take the whole index axis
  [a l] = xcorr (zscore (q), zscore (r), maxlag./dt, 'coeff');

  # consider only positive lags, i.e. delays
  lplus  = l > 0;
  a      = a (lplus);   

  # Discrete maxima
  [maxcorr, delay] = max (a);
  delay            = delay * dt;

  
%!demo
%! t = 0:0.01:5;
%!  function y = f(t,a,b,c)
%!    y = a.*exp(-(t-b).^2/(2*c.^2));
%!  endfunction
%! r = f (t,1,2,0.2);
%! q = f (t,0.5,4,0.6);
%! plot(t,r,'r',t,q,'b')
%! legend('rain signal', 'flow signal')
%! xlabel('time (sample points)')
%! ylabel('rain, flow signal (L, L3/T)')
%! printf (['rain to flow delay = ', num2str(finddelay(q,r)),' samples','\n']);
%! fflush (stdout);
%! #---------------------------------------------------------
%! # the figure window shows a rain and a delayed flow signal
  
endfunction

