## Copyright (C) 2018 - Patrick Troxler
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program. If not, see <http://www.gnu.org/licenses/>.

## Author: Patrick Troxler <patrick-troxler@hotmail.com>
## Created: 2018-06-19

## -*- texinfo -*-
## @defun {[@var{k} @var{Y} @var{covY} @var{state_hist} @var{covstate_hist}]
## =} kalmantrn (@var{k}, @var{inputs}, meas_vec)
##
## KF prediction and update for a training set 
##
## @var{k} structure containing the prepared KF matrices
##
## @var{Y} vector of predictive mean
##
## @var{covY} covariance matrix of predictive mean
##
## @var{state_hist} history of model states
##
## @var{inputs} input matrix (e.g. rain measurements)
##
## @var{meas_vec} matrix containing the measurements
##
## @seealso{kf_predict, kf_update, ekf_predict1, ekf_update1, kalmantst}
## @end defun 

function [k, Y, covY, state_hist, covstate_hist] = kalmantrn (k, ...
    inputs, measurement_vector)

  % whether we record the evolution of the training
  dimstate = size (k.state_estimate, 1);
  [dimout Nt] = size (measurement_vector);
  
  recordhist = false;
  if nargout > 3
    recordhist    = true;
    state_hist    = zeros (dimstate, Nt);
    covstate_hist = zeros (dimstate, dimstate, Nt);
  endif
  
  Y = zeros (Nt, dimout);
  covY = zeros (dimout, dimout, Nt);


  ## dispatch
  # 
  predict_dispatch = struct (
                    'cp', @kf_predict);
  predict = predict_dispatch.(k.kalmantype{1});

  update_dispatch  = struct (
                        'cu', @kf_update, 
                        'eu1', @ekf_update1);
  update = update_dispatch.(k.kalmantype{2});

  measmatrix_array = false;

    % special case: measurement matrix changes at every time step 
  if size (k.measurement_matrix,1) == Nt
    measmatrix_array = true;
  endif
    

  for i=1:Nt
      if isempty (inputs)
        u = [];
      else
        u = inputs(:,i);
      endif  
      if ~isempty (k.H_derivative_state)
      H = k.H_derivative_state (k.state_estimate, k.measurement_matrix);
      else
      H = k.measurement_matrix;
      endif

      [k.state_estimate, k.state_covariance] = predict (
                                                 k.state_estimate, 
                                                 k.state_covariance, 
                                                 k.transition_matrix, 
                                                 k.process_noise,
                                                 k.input_matrix, 
                                                 u);
      if measmatrix_array
        H = k.measurement_matrix(i,:);
      endif
      [k.state_estimate,...
       k.state_covariance,...
       ~,...
       Y(i, :), covY(:,:,i)] = update (k.state_estimate,...
                                         k.state_covariance,...
                                         measurement_vector(:,i),...
                                         H,...
                                         k.measurement_covariance);

      if recordhist
        % Book-keeping
        state_hist(:,i)      = k.state_estimate;
        covstate_hist(:,:,i) = k.state_covariance;
      endif

  endfor % over time steps

endfunction

%!demo
%! disp('None yet!')

