## Copyright (C) 2018 - Patrick Troxler
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program. If not, see <http://www.gnu.org/licenses/>.

## Author: Patrick Troxler <patrick-troxler@hotmail.com>
## Created: 2018-06-19

## -*- texinfo -*-
## @defun {[@var{Pevents} @var{Devents} @var{idr} @var{idd}] =} splitrain (@var{vdata}, @var{dt},
## @var{P_tr_mm}, @var{P_tr_h}, @var{zeroR}, @var{b_} @var{a_}, @var{T})
## Splits rain events based on defined split time (intermittent time between rain signals) and
## buffers (time before and after rain)
##
## Reference:
##  Luca Brocca 2016 
##
## @var{P} matrix with first column corresponding to time (h), second column corresponding to rain (mm/h)
##
## @var{dt} time step (h)
##
## @var{P_tr_mm} minimal rain volume of event to be considered as rain event (mm)
##
## @var{P_tr_h} chosen time between rain signals to split rain event (h)
##
## @var{zeroR} minimum rain signal to be considered (mm)
##
## @var{a_} buffer time after rain event (h)
##
## @var{b_} buffer time after rain event (dt min)
##
## @var{splitT} chosen time between rain signals to split rain event (dt min)
##
## @var{RE} cell array containing time series and rain of rain events 
##
## @var{rV} total rain volume of considered rain events (mm)
##
##@var{T} temperature vector (if T<0, precipitation is snow)
##
## @end defun 

function [Pevents Devents idr idd] = splitrain (P,dt, P_tr_mm, P_tr_h, zeroR, b_, a_, T = [])

i=1; jj=1; k=1; l=1;
Pevents=[];
idr = [];
if isempty(T)
  T = 10.*ones(size(P,1),1); % temperature is positive; (10°C) assumed
endif

while i < size (P, 1)-P_tr_h / dt
  if P(i, 2) > 0 % rain signal is positive
    i_in = i; % index of positive rain signal
    ii = i_in + P_tr_h / dt - 1; 
    while sum (P(ii-P_tr_h / dt+1:ii, 2)) >= zeroR 
      ii = ii + 1; % rain event proceeds if rain signals incoming
      if ii > size (P, 1) 
        break
      endif
    endwhile
  i_fin = ii - 1;
  for iii = i_fin:-1:i_in
    if P(iii, 2) > 0
      i_fin = iii;
      break
    end
  end
  i = i_fin;         
    if sum (P(i_in:i_fin, 2)) > P_tr_mm && i_fin+a_/dt <= size(P,1) &&...
      ~any(T(i_in:i_fin) < 0)
      
      Pev=P(i_in-b_/dt:i_fin+a_/dt, :);
      idr = cat(2, idr, i_in-b_/dt : i_fin+a_/dt); %tmp
      idd = setdiff (jj:i_in, idr);
      if ~isempty(idr)
        Pevents{k} = Pev;
        k  = k+1;
      endif  
      if ~isempty(idd)
        Devents{l} = P(idd, :);
        l  = l+1;
      endif  
      
      jj = i_fin+a_/dt+1;
    endif
  endif
  i = i+1;
endwhile

idd = setdiff (jj:size (P, 1), idr);
Devents{l} = P(idd, :); % get last dry period 
idd = setdiff (1:i, idr); % get all indeces of dry periods together

endfunction